=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-09-07 15:28:14 +0000
+++ bzrlib/branch.py	2005-09-08 00:40:15 +0000
@@ -480,7 +480,7 @@
         """Print `file` to stdout."""
         self.lock_read()
         try:
-            tree = self.revision_tree(self.lookup_revision(revno))
+            tree = self.revision_tree(self.get_rev_id(revno))
             # use inventory as it was in that revision
             file_id = tree.inventory.path2id(file)
             if not file_id:
@@ -897,12 +897,12 @@
         # that they have a revno/revid.
         #   -- lalo@exoweb.net, 2005-09-07
         from bzrlib.errors import NoSuchRevision
-        from bzrlib.revisionspec import get_revision_info
+        from bzrlib.revisionspec import RevisionSpec
         try:
-            revno, info = get_revision_info(self, revision)
+            spec = RevisionSpec(self, revision)
         except NoSuchRevision:
             return None
-        return info
+        return spec.rev_id
 
 
     def revision_id_to_revno(self, revision_id):
@@ -1348,7 +1348,7 @@
         The name of a local directory that exists but is empty.
     """
     from bzrlib.merge import merge
-    from bzrlib.revisionspec import get_revision_info
+    from bzrlib.revisionspec import RevisionSpec
 
     assert isinstance(branch_from, Branch)
     assert isinstance(to_location, basestring)
@@ -1358,7 +1358,7 @@
     if revision is None:
         revno = branch_from.revno()
     else:
-        revno, rev_id = get_revision_info(branch_from, revision)
+        revno, rev_id = RevisionSpec(branch_from, revision)
     br_to.update_revisions(branch_from, stop_revision=revno)
     merge((to_location, -1), (to_location, 0), this_dir=to_location,
           check_clean=False, ignore_zero=True)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-07 15:28:14 +0000
+++ bzrlib/builtins.py	2005-09-08 00:40:15 +0000
@@ -23,7 +23,7 @@
 from bzrlib.trace import mutter, note, log_error, warning
 from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError
 from bzrlib.branch import find_branch
-from bzrlib.revisionspec import get_revision_info
+from bzrlib.revisionspec import RevisionSpec
 from bzrlib import BZRDIR
 from bzrlib.commands import Command
 
@@ -125,7 +125,7 @@
         b = find_branch('.')
 
         for rev in revs:
-            print '%4d %s' % get_revision_info(b, rev)
+            print '%4d %s' % RevisionSpec(b, rev)
 
     
 class cmd_add(Command):
@@ -685,10 +685,10 @@
             rev1 = None
             rev2 = None
         elif len(revision) == 1:
-            rev1 = rev2 = get_revision_info(b, revision[0])[0]
+            rev1 = rev2 = RevisionSpec(b, revision[0]).revno
         elif len(revision) == 2:
-            rev1 = get_revision_info(b, revision[0])[0]
-            rev2 = get_revision_info(b, revision[1])[0]
+            rev1 = RevisionSpec(b, revision[0]).revno
+            rev2 = RevisionSpec(b, revision[1]).revno
         else:
             raise BzrCommandError('bzr log --revision takes one or two values.')
 
@@ -861,7 +861,7 @@
         except ValueError:
             raise BzrCommandError("not a valid revision-number: %r" % revno)
 
-        print find_branch('.').lookup_revision(revno)
+        print find_branch('.').get_rev_id(revno)
 
 
 class cmd_export(Command):
@@ -886,7 +886,7 @@
         else:
             if len(revision) != 1:
                 raise BzrError('bzr export --revision takes exactly 1 argument')
-            revno, rev_id = get_revision_info(b, revision[0])
+            rev_id = RevisionSpec(b, revision[0]).rev_id
         t = b.revision_tree(rev_id)
         root, ext = os.path.splitext(dest)
         if not format:

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-09-07 15:28:14 +0000
+++ bzrlib/commands.py	2005-09-08 00:40:15 +0000
@@ -73,7 +73,7 @@
     each revision.
 
     It supports integers directly, but everything else it
-    defers for passing to revisionspec.get_revision_info()
+    defers for passing to RevisionSpec.
 
     >>> _parse_revision_str('234')
     [234]

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2005-09-07 15:28:14 +0000
+++ bzrlib/revisionspec.py	2005-09-08 00:40:15 +0000
@@ -24,52 +24,88 @@
 # This should match a prefix with a function which accepts it
 REVISION_NAMESPACES = {}
 
-def get_revision_info(branch, spec):
-    """Return (revno, revision id) for revision specifier.
-
-    spec can be an integer, in which case it is assumed to be revno
-    (though this will translate negative values into positive ones)
-    spec can also be a string, in which case it is parsed for something
-    like 'date:' or 'revid:' etc.
-
-    A revid is always returned.  If it is None, the specifier referred to
-    the null revision.  If the revid does not occur in the revision
-    history, revno will be None.
+class RevisionSpec(object):
+    """Equivalent to the old get_revision_info().
+    An instance has two useful attributes: revno, and rev_id.
+
+    They can also be accessed as spec[0] and spec[1] respectively,
+    so that you can write code like:
+    revno, rev_id = RevisionSpec(branch, spec)
+    although this is probably going to be deprecated later.
+
+    Revision specs are an UI element, and they have been moved out
+    of the branch class to leave "back-end" classes unaware of such
+    details.  Code that gets a revno or rev_id from other code should
+    not be using revision specs - revnos and revision ids are the
+    accepted ways to refer to revisions internally.
     """
-
-    if spec is None:
-        return 0, None
-    revno = None
-    try:# Convert to int if possible
-        spec = int(spec)
-    except ValueError:
-        pass
-    revs = branch.revision_history()
-    if isinstance(spec, int):
-        if spec < 0:
-            revno = len(revs) + spec + 1
-        else:
-            revno = spec
-        rev_id = branch.get_rev_id(revno, revs)
-    elif isinstance(spec, basestring):
-        for prefix, func in REVISION_NAMESPACES.iteritems():
-            if spec.startswith(prefix):
-                result = func(branch, revs, spec)
-                if len(result) > 1:
-                    revno, rev_id = result
-                else:
-                    revno = result[0]
-                    rev_id = branch.get_rev_id(revno, revs)
-                break
-        else:
-            raise BzrError('No namespace registered for string: %r' %
-                           spec)
-    else:
-        raise TypeError('Unhandled revision type %s' % spec)
-
-    if revno is None or rev_id is None:
-        raise NoSuchRevision(branch, spec)
-    return revno, rev_id
+    def __init__(self, branch, spec):
+        """Parse a revision specifier.
+
+        spec can be an integer, in which case it is assumed to be revno
+        (though this will translate negative values into positive ones)
+        spec can also be a string, in which case it is parsed for something
+        like 'date:' or 'revid:' etc.
+        """
+        self.branch = branch
+
+        if spec is None:
+            self.revno = 0
+            self.rev_id = None
+            return
+        self.revno = None
+        try:# Convert to int if possible
+            spec = int(spec)
+        except ValueError:
+            pass
+        revs = branch.revision_history()
+        if isinstance(spec, int):
+            if spec < 0:
+                self.revno = len(revs) + spec + 1
+            else:
+                self.revno = spec
+            self.rev_id = branch.get_rev_id(self.revno, revs)
+        elif isinstance(spec, basestring):
+            for prefix, func in REVISION_NAMESPACES.iteritems():
+                if spec.startswith(prefix):
+                    result = func(branch, revs, spec)
+                    if len(result) > 1:
+                        self.revno, self.rev_id = result
+                    else:
+                        self.revno = result[0]
+                        self.rev_id = branch.get_rev_id(self.revno, revs)
+                    break
+            else:
+                raise BzrError('No namespace registered for string: %r' %
+                               spec)
+        else:
+            raise TypeError('Unhandled revision type %s' % spec)
+
+        if self.revno is None or self.rev_id is None:
+            raise NoSuchRevision(branch, spec)
+
+    def __len__(self):
+        return 2
+
+    def __getitem__(self, index):
+        if index == 0: return self.revno
+        if index == 1: return self.rev_id
+        raise IndexError(index)
+
+    def get(self):
+        return self.branch.get_revision(self.rev_id)
+
+    def __eq__(self, other):
+        if type(other) not in (tuple, list, type(self)):
+            return False
+        if type(other) is type(self) and self.branch is not other.branch:
+            return False
+        print 'comparing', tuple(self), tuple(other)
+        return tuple(self) == tuple(other)
+
+    def __repr__(self):
+        return '<bzrlib.revisionspec.RevisionSpec object %s, %s for %r>' % (
+            self.revno, self.rev_id, self.branch)
 
 
 # private API

=== modified file 'bzrlib/selftest/testrevisionnamespaces.py'
--- bzrlib/selftest/testrevisionnamespaces.py	2005-09-07 15:28:14 +0000
+++ bzrlib/selftest/testrevisionnamespaces.py	2005-09-08 00:40:15 +0000
@@ -22,7 +22,7 @@
         """Functional tests for hashcache"""
         from bzrlib.errors import NoSuchRevision
         from bzrlib.branch import Branch
-        from bzrlib.revisionspec import get_revision_info
+        from bzrlib.revisionspec import RevisionSpec
 
         b = Branch('.', init=True)
 
@@ -30,19 +30,15 @@
         b.commit('Commit two', rev_id='a@r-0-2')
         b.commit('Commit three', rev_id='a@r-0-3')
 
-        self.assertEquals(get_revision_info(b, None), (0, None))
-        self.assertEquals(get_revision_info(b, 1), (1, 'a@r-0-1'))
-        self.assertEquals(get_revision_info(b, 'revno:1'), (1, 'a@r-0-1'))
-        self.assertEquals(get_revision_info(b, 'revid:a@r-0-1'), (1, 'a@r-0-1'))
-        self.assertRaises(NoSuchRevision, get_revision_info, b, 'revid:a@r-0-0')
-        self.assertRaises(TypeError, get_revision_info, b, object)
-
-        self.assertEquals(get_revision_info(b, 'date:-tomorrow'), (3, 'a@r-0-3'))
-        self.assertEquals(get_revision_info(b, 'date:+today'), (1, 'a@r-0-1'))
-
-        self.assertEquals(get_revision_info(b, 'last:1'), (3, 'a@r-0-3'))
-        self.assertEquals(get_revision_info(b, '-1'), (3, 'a@r-0-3'))
-
-        os.mkdir('newbranch')
-        b2 = Branch('newbranch', init=True)
-        self.assertEquals(b2.lookup_revision('revid:a@r-0-1'), None)
+        self.assertEquals(RevisionSpec(b, None), (0, None))
+        self.assertEquals(RevisionSpec(b, 1), (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec(b, 'revno:1'), (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec(b, 'revid:a@r-0-1'), (1, 'a@r-0-1'))
+        self.assertRaises(NoSuchRevision, RevisionSpec, b, 'revid:a@r-0-0')
+        self.assertRaises(TypeError, RevisionSpec, b, object)
+
+        self.assertEquals(RevisionSpec(b, 'date:-tomorrow'), (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec(b, 'date:+today'), (1, 'a@r-0-1'))
+
+        self.assertEquals(RevisionSpec(b, 'last:1'), (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec(b, '-1'), (3, 'a@r-0-3'))

